==> ocg_socket-0.0.1.tar.gz: lib/src/centrifuge.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:ocg_app/ocg_app.dart';
import 'package:centrifuge/centrifuge.dart' as centrifuge;
import 'package:flutter/cupertino.dart';
import 'package:http/http.dart' as http;
import 'package:ocg_socket/src/base.dart';
import 'package:ocg_socket/src/models/socket.dart';
import 'package:rxdart/subjects.dart';

class CentrifugeSocket extends BaseSocket {
  final TokenRepository tokenRepository;
  final String url;
  final String subscribeUrl;
  static centrifuge.Client _client;
  final http.BaseClient httpClient;
  final BehaviorSubject<SocketData> _subject;
  final BehaviorSubject<SocketEvent> _eventSubject;
  final Map<String, centrifuge.Subscription> _subscriptions;
  bool _isConnected;

  CentrifugeSocket({
    @required this.url,
    this.subscribeUrl = '/centrifuge/subscribe',
    @required this.tokenRepository,
    @required this.httpClient,
  })  : _subject = BehaviorSubject<SocketData>(),
        _eventSubject = BehaviorSubject<SocketEvent>(),
        _subscriptions = {};

  @override
  Future<void> connect() async {
    if (_client == null) {
      _client = await _createPrivateClient();
      var tokenResponse = await tokenRepository.getToken();
      if (!tokenResponse.isSuccess) {
        return null;
      }
      _client.setConnectData("\"${tokenResponse.data.token}\"".codeUnits);
      _client.connect();
      _isConnected = true;
    }
  }

  Future<centrifuge.Client> _createPrivateClient() async {
    var tokenResponse = await tokenRepository.getToken();
    if (!tokenResponse.isSuccess) {
      return null;
    }
    AppToken token = tokenResponse.data;
    Map<String, dynamic> headers = {"Authorization": "Bearer ${token.token}"};
    centrifuge.Client client = centrifuge.createClient(
      url,
      config: centrifuge.ClientConfig(
          headers: headers..addAll({"Content-Type": 'application/json'}),
          onPrivateSub: (event) async {
            try {
              Map json = {
                "client": event.clientID,
                "channels": [event.channel]
              };
              var request = await httpClient.post(subscribeUrl,
                  body: jsonEncode(json), headers: ApiConfig.headers);
              var responseJson = jsonDecode(request.body);
              if (responseJson is Map) {
                var channelList = responseJson["channels"];
                if (channelList is List) {
                  String token = channelList[0]["token"];
                  return token;
                }
              }
              return null;
            } catch (e) {
              debugPrint("Error $e");
              return null;
            }
          }),
    );
    return client;
  }

  Future<void> subscribe(String channel) async {
    if (_subscriptions[channel] != null) {
      unsubscribe(channel);
    }
    centrifuge.Subscription subscription = _client.getSubscription("$channel");
    _handleSubscription(subscription);
    subscription.subscribe();
    _subscriptions[channel] = subscription;
  }

  Future<void> unsubscribe(String channel) async {
    centrifuge.Subscription subscription = _subscriptions[channel];
    if (subscription != null) {
      subscription.unsubscribe();
      _subscriptions[channel] = null;
    }
  }

  void _handleSubscription(centrifuge.Subscription subscription) async {
    subscription.publishStream.listen((val) async {
      var json = jsonDecode(utf8.decode(val.data));
      String type = json["type"];
      var data = json;
      SocketData socketData = SocketData.named(type: type, data: data);
      _subject.add(socketData);
    });
    subscription.joinStream.listen((val) {
      _eventSubject.add(SocketJoinEvent(val.client, val.user));
    });
    subscription.leaveStream.listen((val) {
      _eventSubject.add(SocketLeaveEvent(val.client, val.user));
    });
    subscription.subscribeSuccessStream.listen((event) {
      _eventSubject.add(SocketSubscribeEvent(subscription.channel));
    });
    subscription.unsubscribeStream.listen((val) {
      _eventSubject.add(SocketUnsubscribeEvent(subscription.channel));
    });
    subscription.subscribeErrorStream.listen((val) {
      _eventSubject.add(SocketErrorEvent(subscription.channel, val.message));
    });
  }

  Stream<SocketData> get stream => _subject.stream;

  Stream<SocketEvent> get eventStream => _eventSubject.stream;

  @override
  Future<void> disconnect() async {
    _subject.close();
    _eventSubject.close();
    _client?.disconnect();
    _client = null;
  }

  @override
  Future<bool> get isConnected async {
    return _isConnected ?? false;
  }
}

==> ocg_socket-0.0.2.tar.gz: lib/src/centrifuge.dart <==
import 'dart:async';
import 'dart:convert';

import 'package:ocg_app/ocg_app.dart';
import 'package:centrifuge/centrifuge.dart' as centrifuge;
import 'package:flutter/cupertino.dart';
import 'package:http/http.dart' as http;
import 'package:ocg_socket/src/base.dart';
import 'package:ocg_socket/src/models/socket.dart';
import 'package:rxdart/subjects.dart';

class CentrifugeSocket extends BaseSocket {
  final TokenRepository tokenRepository;
  final String url;
  final String subscribeUrl;
  static centrifuge.Client _client;
  final http.BaseClient httpClient;
  final BehaviorSubject<SocketData> _subject;
  final BehaviorSubject<SocketEvent> _eventSubject;
  final Map<String, centrifuge.Subscription> _subscriptions;
  SocketState _state;

  CentrifugeSocket({
    @required this.url,
    this.subscribeUrl = '/centrifuge/subscribe',
    @required this.tokenRepository,
    @required this.httpClient,
  })  : _subject = BehaviorSubject<SocketData>(),
        _eventSubject = BehaviorSubject<SocketEvent>(),
        _subscriptions = {},
        _state = SocketState.Off;

  @override
  Future<bool> connect() async {
    if (_state == SocketState.Off) {
      _state = SocketState.Connecting;
      _client = await _createPrivateClient();
      if (_client == null) {
        _state = SocketState.Off;
        return false;
      }
      var tokenResponse = await tokenRepository.getToken();
      if (!tokenResponse.isSuccess) {
        return false;
      }
      _client.setConnectData("\"${tokenResponse.data.token}\"".codeUnits);
      _client.connect();
      _state = SocketState.Connected;
      return true;
    }
    debugPrint("Socket State $_state");
    return _state == SocketState.Connected;
  }

  Future<centrifuge.Client> _createPrivateClient() async {
    var tokenResponse = await tokenRepository.getToken();
    if (!tokenResponse.isSuccess) {
      return null;
    }
    AppToken token = tokenResponse.data;
    Map<String, dynamic> headers = {"Authorization": "Bearer ${token.token}"};
    centrifuge.Client client = centrifuge.createClient(
      url,
      config: centrifuge.ClientConfig(
          headers: headers..addAll({"Content-Type": 'application/json'}),
          onPrivateSub: (event) async {
            try {
              Map json = {
                "client": event.clientID,
                "channels": [event.channel]
              };
              var request = await httpClient.post(subscribeUrl,
                  body: jsonEncode(json), headers: ApiConfig.headers);
              var responseJson = jsonDecode(request.body);
              if (responseJson is Map) {
                var channelList = responseJson["channels"];
                if (channelList is List) {
                  String token = channelList[0]["token"];
                  return token;
                }
              }
              return null;
            } catch (e) {
              debugPrint("Error $e");
              return null;
            }
          }),
    );
    return client;
  }

  Future<void> subscribe(String channel) async {
    if (_subscriptions[channel] != null) {
      await unsubscribe(channel);
    }
    centrifuge.Subscription subscription = _client.getSubscription("$channel");
    _handleSubscription(subscription);
    subscription.subscribe();
    debugPrint("Subscribe $channel");
    _subscriptions[channel] = subscription;
  }

  Future<void> unsubscribe(String channel) async {
    centrifuge.Subscription subscription = _subscriptions[channel];
    if (subscription != null) {
      subscription.unsubscribe();
      _subscriptions[channel] = null;
    }
  }

  void _handleSubscription(centrifuge.Subscription subscription) async {
    subscription.publishStream.listen((val) async {
      var json = jsonDecode(utf8.decode(val.data));
      String type = json["type"];
      var data = json;
      SocketData socketData = SocketData.named(type: type, data: data);
      _subject.add(socketData);
    });
    subscription.joinStream.listen((val) {
      debugPrint("Joined ${val.client}");
      _eventSubject.add(SocketJoinEvent(val.client, val.user));
    });
    subscription.leaveStream.listen((val) {
      debugPrint("Left ${val.client}");
      _eventSubject.add(SocketLeaveEvent(val.client, val.user));
    });
    subscription.subscribeSuccessStream.listen((event) {
      debugPrint("Subscribed");
      _eventSubject.add(SocketSubscribeEvent(subscription.channel));
    });
    subscription.unsubscribeStream.listen((val) {
      debugPrint("Unsubscribed");
      _eventSubject.add(SocketUnsubscribeEvent(subscription.channel));
    });
    subscription.subscribeErrorStream.listen((val) {
      debugPrint("Subscribe Error ${val.message}");
      _eventSubject.add(SocketErrorEvent(subscription.channel, val.message));
    });
  }

  Stream<SocketData> get stream => _subject.stream;

  Stream<SocketEvent> get eventStream => _eventSubject.stream;

  @override
  Future<bool> disconnect() async {
    _subject.close();
    _eventSubject.close();
    _client?.disconnect();
    _client = null;
    return true;
  }

  @override
  SocketState get connectedState {
    return _state ?? SocketState.Off;
  }
}
